////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     GFUN Module Registers
///
///     This file defines the GFUN registers. These registers MUST match the
///     GFUN hardware specs.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

// ---- Constants and Types -------------------------------------------------

//
// GFUN register group base addresses & sizes
//

#define GFUN_REG_GLOB_BASE  (0x0000)
#define GFUN_REG_GLOB_SIZE  (0x0200)
#define GFUN_REG_MBOX_BASE  (0x0200)
#define GFUN_REG_MBOX_SIZE  (0x0200)
#define GFUN_REG_CH_BASE    (0x0400)
#define GFUN_REG_CH_SIZE    (0x0040)

//
// GFUN simple mailbox message registers
//

#define mGFUN_REG_MBOX_ADDR(offset)  (GFUN_REG_MBOX_BASE + (offset))

#define mGFUN_REG_MBOX_SLOT(n)       mGFUN_REG_MBOX_ADDR((n) * 0x4)

//
// GFUN global registers
//

#define mGFUN_REG_GLOB_ADDR(n)             (GFUN_REG_GLOB_BASE + (n))

#define GFUN_REG_GLOB_FUNCSTAT             mGFUN_REG_GLOB_ADDR(0x0000)
#define GFUN_REG_GLOB_FUNCCAP              mGFUN_REG_GLOB_ADDR(0x0004)
#define GFUN_REG_GLOB_MSTRINTRSTAT         mGFUN_REG_GLOB_ADDR(0x0008)
#define GFUN_REG_GLOB_OUTDMADISINTRSTAT    mGFUN_REG_GLOB_ADDR(0x000C)
#define GFUN_REG_GLOB_OUTIOCINTRSTAT       mGFUN_REG_GLOB_ADDR(0x0010)
#define GFUN_REG_GLOB_INDMADISINTRSTAT     mGFUN_REG_GLOB_ADDR(0x0014)
#define GFUN_REG_GLOB_INIOCINTRSTAT        mGFUN_REG_GLOB_ADDR(0x0018)
#define GFUN_REG_GLOB_INTREN               mGFUN_REG_GLOB_ADDR(0x001C)
#define GFUN_REG_GLOB_OUTDMADISINTREN      mGFUN_REG_GLOB_ADDR(0x0020)
#define GFUN_REG_GLOB_OUTIOCINTREN         mGFUN_REG_GLOB_ADDR(0x0024)
#define GFUN_REG_GLOB_INDMADISINTREN       mGFUN_REG_GLOB_ADDR(0x0028)
#define GFUN_REG_GLOB_INIOCINTREN          mGFUN_REG_GLOB_ADDR(0x002C)
#define GFUN_REG_GLOB_DRIVERMSGSEMAPHORE   mGFUN_REG_GLOB_ADDR(0x0030)
#define GFUN_REG_GLOB_DRIVERMUTEX          mGFUN_REG_GLOB_ADDR(0x0034)
#define GFUN_REG_GLOB_IOCPERIOD            mGFUN_REG_GLOB_ADDR(0x0038)
#define GFUN_REG_GLOB_GLOBCNTLSTAT         mGFUN_REG_GLOB_ADDR(0x003C)
#define GFUN_REG_GLOB_GLOBOUTDMACNTRLSTAT  mGFUN_REG_GLOB_ADDR(0x0040)
#define GFUN_REG_GLOB_GLOBINDMACNTRLSTAT   mGFUN_REG_GLOB_ADDR(0x0044)

// GFUN_REG_GLOB_FUNCSTAT masks
#define GFUN_REG_GLOB_FUNCSTAT_FUNC_RDY_MASK        (0x00000001)

// GFUN_REG_GLOB_GLOBCNTLSTAT masks
#define GFUN_REG_GLOB_GLOBCNTLSTAT_INIT_MASK        (1 << 0)
#define GFUN_REG_GLOB_GLOBCNTLSTAT_INIT_ALIAS_MASK  (1 << 1)

//
// GFUN channel registers
//

#define mGFUN_REG_CH_ADDR(ch, n)           (GFUN_REG_CH_BASE + (n) + \
                                            ((ch) * GFUN_REG_CH_SIZE))

#define GFUN_REG_CH_OUTDMACNTRLSTAT(ch)    mGFUN_REG_CH_ADDR(ch, 0x0000)
#define GFUN_REG_CH_OUTDMACHNLDMAEN(ch)    mGFUN_REG_CH_ADDR(ch, 0x0004)
#define GFUN_REG_CH_OUTDESCRLEN(ch)        mGFUN_REG_CH_ADDR(ch, 0x000C)
#define GFUN_REG_CH_OUTDESCRBASELO(ch)     mGFUN_REG_CH_ADDR(ch, 0x0010)
#define GFUN_REG_CH_OUTDESCRBASEHI(ch)     mGFUN_REG_CH_ADDR(ch, 0x0014)
#define GFUN_REG_CH_OUTDESCRHEADPTR(ch)    mGFUN_REG_CH_ADDR(ch, 0x0018)
#define GFUN_REG_CH_OUTDESCRTAILPTR(ch)    mGFUN_REG_CH_ADDR(ch, 0x001C)
#define GFUN_REG_CH_INDMACNTRLSTAT(ch)     mGFUN_REG_CH_ADDR(ch, 0x0020)
#define GFUN_REG_CH_INDMACHNLDMAEN(ch)     mGFUN_REG_CH_ADDR(ch, 0x0024)

// Note: The following registers are located at different offsets in
//       TERA1 versus TERA2, which is why the macro requires the caller
//       to supply the device type variable.
#define GFUN_REG_CH_INDESCRLEN(ch, type)         mGFUN_REG_CH_ADDR(ch, (0x0028 + ((type)*4)))
#define GFUN_REG_CH_INDESCRBASELO(ch, type)      mGFUN_REG_CH_ADDR(ch, (0x002C + ((type)*4)))
#define GFUN_REG_CH_INDESCRBASEHI(ch, type)      mGFUN_REG_CH_ADDR(ch, (0x0030 + ((type)*4)))
#define GFUN_REG_CH_INDESCRHEADPTR(ch, type)     mGFUN_REG_CH_ADDR(ch, (0x0034 + ((type)*4)))
#define GFUN_REG_CH_INDESCRTAILPTR(ch, type)     mGFUN_REG_CH_ADDR(ch, (0x0038 + ((type)*4)))

// GFUN_REG_CH_OUTDMACNTRLSTAT masks
#define GFUN_REG_CH_OUTDMACNTRLSTAT_WB_DIS_MASK        (0x00000010)

// GFUN_REG_CH_OUTDMACHNLDMAEN masks
#define GFUN_REG_CH_OUTDMACHNLDMAEN_DMA_EN_MASK        (0x00000001)
#define GFUN_REG_CH_OUTDMACHNLDMAEN_DMA_EN_ALIAS_MASK  (0x00000010)

// GFUN_REG_CH_OUTDESCRLEN masks
#define GFUN_REG_CH_OUTDESCRLEN_DESCR_Q_LEN_MASK       (0x000FFFE0)

// GFUN_REG_CH_OUTDESCRHEADPTR masks
#define GFUN_REG_CH_OUTDESCRHEADPTR_DESCR_HEAD_MASK    (0x000FFFE0)

// GFUN_REG_CH_OUTDESCRTAILPTR masks
#define GFUN_REG_CH_OUTDESCRTAILPTR_DESCR_TAIL_MASK    (0x000FFFE0)

// GFUN_REG_CH_INDMACNTRLSTAT masks
#define GFUN_REG_CH_INDMACNTRLSTAT_WB_DIS_MASK         (0x00000010)

// GFUN_REG_CH_INDMACHNLDMAEN masks
#define GFUN_REG_CH_INDMACHNLDMAEN_DMA_EN_MASK         (0x00000001)
#define GFUN_REG_CH_INDMACHNLDMAEN_DMA_EN_ALIAS_MASK   (0x00000010)

// GFUN_REG_CH_INDESCRLEN masks
#define GFUN_REG_CH_INDESCRLEN_DESCR_Q_LEN_MASK        (0x000FFFE0)

// GFUN_REG_CH_INDESCRHEADPTR masks
#define GFUN_REG_CH_INDESCRHEADPTR_DESCR_HEAD_MASK     (0x000FFFE0)

// GFUN_REG_CH_INDESCRTAILPTR masks
#define GFUN_REG_CH_INDESCRTAILPTR_DESCR_TAIL_MASK     (0x000FFFE0)

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

