////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Packet Module
///
///     This file defines the packet module API prototypes and definitions.
///     This module uses memory buffers from the memory module.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_mem.h"

// ---- Constants and Types -------------------------------------------------

// --------------------------------------------------------------------------
//  sTERA_PKT
///     Packet structure
//
//  Description:
///     This structure defines a packet. The only variables that should be
///     accessed outside of this module is the buffer pointer and buffer length.
///     All other members are internal to this module and should NOT be
///     modified.
///
typedef struct sTERA_PKT_STRUCT
{
    struct sTERA_PKT_STRUCT *next;        ///< Next pointer
    sTERA_MEM_BUF           *buf;         ///< Pointer to packet buffer
    UINT32                   buf_len;     ///< Length of packet buffer
} sTERA_PKT;

// --------------------------------------------------------------------------
//  sTERA_PKT_Q
///     Packet queue
//
//  Description:
///     This structure defines a packet queue. Each packet queue needs to be
///     initialized before it can be used.
///
typedef struct sTERA_PKT_Q_STRUCT
{
    INT         initialized;    ///< Initialized flag
    struct semaphore  lock;     ///< Packet queue synchronization lock
    sTERA_PKT  *head;           ///< Pointer to head of packet queue
    UINT32      enqueue_cnt;    ///< Enqueue counter
    UINT32      dequeue_cnt;    ///< Dequeue counter
} sTERA_PKT_Q;

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

extern INT32 tera_pkt_init(
    struct pci_dev *dev,
    UINT32          dev_id
    );

extern void tera_pkt_cleanup(
    struct pci_dev *dev
    );

extern sTERA_PKT *tera_pkt_alloc(
    struct pci_dev *dev,
    UINT32          buf_len,
    UINT32          num_pkt
    );

extern void tera_pkt_free(
    struct pci_dev *dev,
    sTERA_PKT      *pkt
    );

extern INT32 tera_pkt_q_init(
    sTERA_PKT_Q *pkt_q
    );

extern void tera_pkt_q_purge(
    struct pci_dev *dev,
    sTERA_PKT_Q    *pkt_q
    );

extern void tera_pkt_q_enqueue(
    sTERA_PKT_Q *pkt_q,
    sTERA_PKT   *pkt
    );

extern void tera_pkt_q_head_enqueue(
    sTERA_PKT_Q *pkt_q,
    sTERA_PKT   *pkt
    );

extern sTERA_PKT *tera_pkt_q_dequeue(
    sTERA_PKT_Q *pkt_q
    );

