////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Device Access Class
///
///     This file defines the device access class and related structures.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_types.h"

// ---- Constants and Types -------------------------------------------------

// --------------------------------------------------------------------------
//  cTERA_DEV
///     Device access
//
//  Description:
///     This class defines a device access object to allow I/O control (ioctl)
///     communication with a device in kernel space. This class simplifies the
///     calling of synchronous and asynchronous ioctls.
///
class cTERA_DEV
{

public:

    // --- Public Functions -------------------------------------------------

    cTERA_DEV(
        const char *dev_path = 0);

    ~cTERA_DEV(void);

    INT is_ready(void) const;

    INT32 init(
        const char *dev_path);

    INT32 dev_fd(void) const
    {
        return m_dev_fd;
    }

    INT32 synch_ioctl(
        UINT32 ioctl,
        void  *buf,
        UINT32 buf_len,
        UINT32 timeout);

protected:

    // --- Protected Functions ----------------------------------------------

    // --- Protected Variables ----------------------------------------------

#define MAX_PATH 256

    INT32   m_dev_fd;                ///< Device file descriptor
    bool    m_initialized;           ///< Initialized flag
    bool    m_ready;                 ///< Ready flag
    char    m_dev_path[MAX_PATH];    ///< Path to device

}; // cTERA_DEV

