////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     PCoIP Host Software Features Property Page Class
///
///     This file defines the PCoIP Host Software Features property page
///     class.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_pcoip_agent_gui_res.h"

// ---- Constants and Types -------------------------------------------------

// --------------------------------------------------------------------------
//  cPCOIP_AGENT_GUI_PROP_FEAT
///     Features property page
//
//  Description:
///     This class defines the PCoIP Host Software Features property page.
///     This property page allows the user to configure options for the agent,
///     such as:
///         - enable/disable lock workstation option
///         - enable/disable local cursor and keyboard
///         - enable/disable Wake-on-LAN option
///
class cPCOIP_AGENT_GUI_PROP_FEAT : public cPCOIP_AGENT_GUI_PROP_FEAT_BASE
{

public:

    // --- Public Functions -------------------------------------------------

    cPCOIP_AGENT_GUI_PROP_FEAT(
        wxWindow *parent);

    ~cPCOIP_AGENT_GUI_PROP_FEAT(void);

    INT32 lcur_spd_get(void);

    bool lcur_spd_enable_get(void);

    UINT32 cur_visibility_get_reg(void);

    UINT16 lkbd_delay_get(void);

    UINT16 lkbd_rate_get(void);

    void lkbd_delay_set(UINT16 delay);

    void lkbd_rate_set(UINT16 rate);

    void client_kbd_settings_avail_set(INT32 status);

    void init_dialog_lcur_lkbd(void);

    // --- Public Variables -------------------------------------------------

protected:

    // --- Protected Functions ----------------------------------------------

    // Event handlers
    void evt_init_dialog(
        wxInitDialogEvent& evt);

    void evt_close(
        wxCloseEvent& evt);

    void evt_update_ui(
        wxUpdateUIEvent& evt);

    void evt_any_btn(
        wxCommandEvent& evt);

    void evt_apply_btn(
        wxCommandEvent& evt);

    void evt_lcur_lkbd_en_chk(
        wxCommandEvent& evt);

    void evt_lcur_vis_choice(
        wxCommandEvent& evt);

    void evt_any(
        wxCommandEvent& evt);

    void evt_any_scroll(
        wxScrollEvent& evt);

    void init_dialog_security(void);

    void init_lcur_slider(bool enable);

    void init_lkbd_sliders(bool enable);

    void lcur_lkbd_ctrl_update(void);

    void lcur_hlp_btn_setup(
        int display_help);

    void lkbd_delay_set_gui(UINT16 client_delay);

    void lkbd_rate_set_gui(UINT16 client_rate);

    UINT16 lkbd_delay_get_gui(void);

    UINT16 lkbd_rate_get_gui(void);

    // --- Protected Variables ----------------------------------------------

    bool        m_modified;                ///< Settings modified flag
    INT         m_lkbd_delay_slider_set;   ///< flag used to determine whether the slider can be read
    INT         m_lkbd_rate_slider_set;    ///< flag used to determine whether the slider can be read
    UINT32      m_lkbd_delay;              ///< Variable that tracks the value of the kbd repeat delay
    UINT32      m_lkbd_rate;               ///< Variable that tracks the value of the kbd repeat rate
    INT32       m_client_kbd_setting_avail;///< indicates whether the client supports reading repeat rate

    DECLARE_EVENT_TABLE()
}; // cPCOIP_AGENT_GUI_PROP_FEAT

// ---- Variable Externs ----------------------------------------------------

// ---- Function Prototypes -------------------------------------------------

