////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     PCoIP Host Software Application Class
///
///     This file defines the PCoIP Host Software application class.
///
////////////////////////////////////////////////////////////////////////////

#pragma once

// ---- Include Files -------------------------------------------------------

#include "tera_pcoip_agent.h"

// ---- Constants and Types -------------------------------------------------

// Command line arguments
#define PCOIP_AGENT_CMD_LINE_ARG_SVC_INSTALL    L"-service_install"
#define PCOIP_AGENT_CMD_LINE_ARG_SVC_UNINSTALL  L"-service_uninstall"
#define PCOIP_AGENT_CMD_LINE_ARG_SVC_RUN        L"-service_run"
#define PCOIP_AGENT_CMD_LINE_ARG_SVC_START      L"-service_start"
#define PCOIP_AGENT_CMD_LINE_ARG_SVC_LAUNCH     L"-service_launch"
#define PCOIP_AGENT_CMD_LINE_ARG_UNINSTALL      L"-uninstall"
#define PCOIP_AGENT_CMD_LINE_ARG_LOGFILE        L"-logfile"

// Global shutdown event (used by Windows Service and User Application)
#define PCOIP_AGENT_EVENT_SHUTDOWN              L"Global\\PcoipAgentShutdown"

// tooltip strings
#define IDS_STR_HELP_GEN_NO_PCOIP_SESS     _T("A PCoIP session must be active to use this feature\nPossible causes:\n  - session disconnected\n  - a session is not active\nPossible solutions:\n  - establish a PCoIP session")
#define IDS_STR_HELP_BAD_SCREEN_CFG        _T("Invalid monitor configuration\nPossible solutions:\n  - select Monitors tab & configure a monitor for each screen")
#define IDS_STR_HELP_LCUR_FF00             _T("Invalid monitor configuration\nPossible solutions:\n  - select Monitors tab & configure a monitor for each screen\n  - update firmware to a version compatible with this software")
#define IDS_STR_HELP_LCUR_FF01             _T("Invalid monitor configuration\nPossible solutions:\n  - select Monitors tab & configure a monitor for each screen\n  - update firmware to a version compatible with this software")
#define IDS_STR_HELP_LCUR_FF02             _T("Display topology mismatch\nPossible solutions:\n  - select Monitors tab & configure a monitor for each screen\n  - verify monitors reported on the zero client\n    Info=>Attached Devices web page match the\n    active GPU monitors\n  - verify client display topology resolution(s) match\n    X Window screen resolution(s)\n  - close/exit the pcoip_agent executable, delete the file\n    ~/.pcoip_agent and restart the pcoip_agent executable\n  - restart the host PC\n  - update firmware to a version compatible with this software")
#define IDS_STR_HELP_LCUR_0001             _T("No match for host monitor\nPossible causes:\n  - host was connected previously with another client\n  - monitors have changed on client\n  - new monitors plugged in without a power cycle of host PC\nPossible solutions:\n  - restart the host PC\n  - disconnect and reconnect the session")
#define IDS_STR_HELP_LCUR_0002             _T("Mismatched Vendor ID field in the monitor information")
#define IDS_STR_HELP_LCUR_0003             _T("Mismatched Product ID field in the monitor information")
#define IDS_STR_HELP_LCUR_0004             _T("Mismatched Serial Number field in the monitor information")
#define IDS_STR_HELP_ZERO_MON              _T("No monitors detected\nPossible causes:\n  - PCoIP session disconnected or not active\n  - RDP session active\n  - firmware does not support reporting monitor info\nPossible solutions:\n  - establish a PCoIP session\n  - update firmware to a version compatible with this software")
#define IDS_STR_HELP_GEN_NO_SUPPORT        _T("Feature not supported\nPossible causes:\n  - firmware does not support this feature\n  - PCoIP host kernel driver not loaded\n  - configuration file error\nPossible solutions:\n  - update firmware to a version compatible with this software\n  - restart pcoip_agent executable\n  - delete ~/.pcoip_agent and restart pcoip_agent executable")
#define IDS_STR_HELP_GEN_NO_LINK           _T("No link to the PCoIP device\nPossible causes:\n  - PCoIP host kernel driver not loaded\n  - 'Host Driver' option not enabled on host card\n  - driver/software not compatible with firmware")
#define IDS_STR_HELP_LCUR_FF03             _T("No connected monitors on the client\nPossible causes:\n  - monitors were plugged in after client powered on\nPossible solutions:\n  - power cycle the client\n  - disconnect and reconnect session")
#define IDS_STR_HELP_WOL_0001              _T("Waiting for selected network interface to obtain an IP address\nPossible causes:\n  - DHCP lease is being renewed\n  - static IP address has been changed\n  - cable/wireless link has been disconnected")
#define IDS_STR_HELP_WOL_0002              _T("Cannot detect any Ethernet/wireless network interfaces")
#define IDS_STR_HELP_STAT_GET_ERR          _T("Error reading session statistics\nPossible solutions:\n  - restart PCoIP host software\n  - disconnect and reconnect session\n  - restart workstation")
#define IDS_STR_HELP_MON_CACHE_TOPO_ERR    _T("Error reading client topology\nPossible solutions:\n  - restart PCoIP host software\n  - disconnect and reconnect session\n  - restart workstation\n  - enable client display topology configuration\n  - verify client display topology resolution(s)\n    match X Window screen resolution(s)\n  - close/exit the pcoip_agent executable, delete the file\n    ~/.pcoip_agent and restart the pcoip_agent executable")
#define IDS_STR_HELP_MON_SCREEN_POS_ERR    _T("Invalid screen configuration data or unsupported X Windows\nconfiguration preventing the host from reading the position\nof each screen.\nPossible solutions:\n  - Close/exit the pcoip_agent executable, delete the file\n    ~/.pcoip_agent and restart the pcoip_agent executable\n  - Enable Xinerama or TwinView")

// ---- Variable Externs ----------------------------------------------------

DECLARE_APP(cTERA_PCOIP_AGENT)

// ---- Function Prototypes -------------------------------------------------

