////////////////////////////////////////////////////////////////////////////
//   Copyright 2013  Teradici Corporation
//
//   This program is free software; you can redistribute it and/or modify it
//   under the terms and conditions of the GNU General Public License,
//   version 2, as published by the Free Software Foundation.
//
////////////////////////////////////////////////////////////////////////////
/// \file
/// \brief
///     Single Instance Class Implementation
///
///     This file implements the single instance class. This class is used to
///     limit the number of instances of an application to one.
///
////////////////////////////////////////////////////////////////////////////

// ---- Include Files -------------------------------------------------------

#include <wx/wxprec.h>

#if !defined(WX_PRECOMP)
    #include <wx/wx.h>
#endif

#include <wx/file.h>

#include "tera_log.h"
#include "tera_sinst.h"

// ---- Global Variables ----------------------------------------------------

// ---- Private Constants and Types -----------------------------------------

// ---- Local Variables -----------------------------------------------------

// ---- Local Function Prototypes -------------------------------------------

// --------------------------------------------------------------------------
//  cTERA_SINST::cTERA_SINST
///     Constructor
//
//  Description:
///     This function is the contructor for this class. This function tries to
///     create a file mapping with the unique ID supplied, and also registering
///     a windows message. The file mapping will allow this instance to share
///     data with other instances.
///
/// \return
///     None
///
cTERA_SINST::cTERA_SINST(
    const wxString& name,
    const wxString& path
    )
{
    bool     err;
    wxString filename = path;

    mTERA_LOG_DBG("start");

    if (filename.empty())
    {
        filename = wxGetHomeDir();
    }

    if (filename.Last() != _T('/'))
    {
        filename += _T('/');
    }

    filename << name;

    // Suppress log since it pops up a dialog when there is a stale lock file
    wxLogNull *suppress_log = new wxLogNull;

    err = Create(name, path);

    if (suppress_log)
    {
        delete suppress_log;
    }

    // Handle the case where it fails
    if (IsAnotherRunning())
    {
        wxFile file(filename, wxFile::read);
        if (file.IsOpened())
        {
            char    buf[256];
            ssize_t count = file.Read(buf, sizeof(buf));
            pid_t   pid;

            if (count != wxInvalidOffset)
            {
                if (sscanf(buf, "%d", (int *)&pid) == 1)
                {
                    mTERA_LOG_DBG("pid %d", pid);

                    //TODO Open proc file as file and read command name from it
                }
            }
        }
    }

    mTERA_LOG_DBG("end");

} // cTERA_SINST::cTERA_SINST()
